﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.Design.Behavior;

namespace AircraftCarrier
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningAircraftCarrier
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityAircraftCarrier AircraftCarrier { get; protected set; }
        /// <summary>
        /// Левая координата отрисовки автомобиля
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки автомобиля
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeigth = null;
        // <summary>
        /// Ширина отрисовки самолета
        /// </summary>
        protected readonly int _aircraftcarrierWidth = 230;
        protected readonly int _aircraftcarrierHeight = 80;
        public DrawningAircraftCarrier(int speed, float weight, Color bodyColor)
        {
            AircraftCarrier = new EntityAircraftCarrier(speed, weight, bodyColor);
        }
        protected DrawningAircraftCarrier(int speed, float weight, Color bodyColor, int aircraftcarrierWidth, int aircraftcarrierHeight) : this(speed, weight, bodyColor)
        {
            _aircraftcarrierWidth = aircraftcarrierWidth;
            _aircraftcarrierHeight = aircraftcarrierHeight;
        }
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x >= 0 && y >= 0 && (x + _aircraftcarrierWidth) <= width && (y + _aircraftcarrierHeight) <= height)
            {
                _startPosX = x;
                _startPosY = y;

                _pictureWidth = width;
                _pictureHeigth = height;
            }
        }
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeigth.HasValue)
            {
                return;
            }
            switch (direction)
            {
                case Direction.Right:
                    if (_startPosX + _aircraftcarrierWidth + AircraftCarrier.Step < _pictureWidth)
                    {
                        _startPosX += AircraftCarrier.Step;
                    }
                    break;
                case Direction.Left:
                    if (_startPosX - AircraftCarrier.Step > 0)
                    {
                        _startPosX -= AircraftCarrier.Step;
                    }
                    break;
                case Direction.Up:
                    if (_startPosY - AircraftCarrier.Step > 0)
                    {
                        _startPosY -= AircraftCarrier.Step;
                    }
                    break;
                case Direction.Down:
                    if (_startPosY + _aircraftcarrierHeight + AircraftCarrier.Step < _pictureHeigth)
                    {
                        _startPosY += AircraftCarrier.Step;
                    }
                    break;
            }
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0 || !_pictureHeigth.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Point PointFront_1 = new Point((int)_startPosX + 10, (int)_startPosY);//основа
            Point PointFront_2 = new Point((int)_startPosX + 170, (int)_startPosY);
            Point PointFront_3 = new Point((int)_startPosX + 230, (int)_startPosY + 40);
            Point PointFront_4 = new Point((int)_startPosX + 170, (int)_startPosY + 80);
            Point PointFront_5 = new Point((int)_startPosX + 10, (int)_startPosY + 80);
            Point[] PointsFront = { PointFront_1, PointFront_2, PointFront_3, PointFront_4, PointFront_5 };
            g.DrawPolygon(pen, PointsFront);
            Brush brBlack = new SolidBrush(Color.Black);//наружние блоки
            g.FillRectangle(brBlack, _startPosX, _startPosY + 20, 10, 15);
            g.FillRectangle(brBlack, _startPosX, _startPosY + 55, 10, 15);
            Brush br = new SolidBrush(AircraftCarrier?.BodyColor ?? Color.Black);//внутренние блоки
            g.FillRectangle(br, _startPosX + 130, _startPosY + 24, 20, 30);
            g.FillRectangle(br, _startPosX + 110, _startPosY + 36, 20, 10);
            g.FillEllipse(br, _startPosX + 150, _startPosY + 27, 32, 30);
        }
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeigth = height;
            if (_pictureWidth <= _aircraftcarrierWidth || _pictureHeigth <= _aircraftcarrierHeight)
            {
                _pictureWidth = null;
                _pictureHeigth = null;
                return;
            }
            if (_startPosX + _aircraftcarrierWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _aircraftcarrierWidth;
            }

            if (_startPosY + _aircraftcarrierHeight > _pictureHeigth)
            {
                _startPosY = _pictureHeigth.Value - _aircraftcarrierHeight;
            }
        }
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosY, _startPosX + _aircraftcarrierWidth, _startPosY + _aircraftcarrierHeight);
        }
        public void ReturnColor(Color returnColor)
        {
            if (AircraftCarrier is not EntityModernAircraftCarrier modernAircraftCarrier)
            {
                return;
            }
            modernAircraftCarrier.DopColor = returnColor;
        }
    }
}