﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.DataFormats;

namespace AircraftCarrier
{
    public partial class FormMapWithSetAircraftCarriers : Form
    {
        private MapWithSetAircraftCarriersGeneric<DrawningObjectAircraftCarrier, AbstractMap>
_mapAircraftCarriersCollectionGeneric;
        /// <summary>
        /// Конструктор
        /// </summary>
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
 {
 { "Простая карта", new SimpleMap() },
 { "Сложная карта", new ComplexMap() }
 };
        private readonly MapsCollection _mapsCollection;
        private readonly ILogger _logger;
        public FormMapWithSetAircraftCarriers(ILogger<FormMapWithSetAircraftCarriers> logger)
        {
            InitializeComponent();
            _logger = logger;
            _mapsCollection = new MapsCollection(pictureBox.Width,
pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;
            listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }
            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >=
           listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index <
           listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }
        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 ||
           string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation("Не все данные были заполнены при добавлении карты");
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK,
               MessageBoxIcon.Error);
                _logger.LogInformation($"Нет карты с названием: {comboBoxSelectorMap.Text}");
                return;
            }
            if (textBoxNewMapName.Text.Contains('|') || textBoxNewMapName.Text.Contains(':') || textBoxNewMapName.Text.Contains(';'))
            {
                MessageBox.Show("Присутствуют недопустимые символы", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation($"В названии карты присутствуют недопустимые символы: {textBoxNewMapName.Text}");
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text,
           _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation($"Добавлена карта {textBoxNewMapName.Text}");
        }
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image =
           _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation($"Переход на карту: {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
        }
        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?",
           "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ??
               string.Empty);
                ReloadMaps();
                _logger.LogInformation($"Удалена карта {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
            }
        }
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    map = new SimpleMap();
                    break;
                case "Сложная карта":
                    map = new ComplexMap();
                    break;
            }
            if (map != null)
            {
                _mapAircraftCarriersCollectionGeneric = new
               MapWithSetAircraftCarriersGeneric<DrawningObjectAircraftCarrier, AbstractMap>(
                pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapAircraftCarriersCollectionGeneric = null;
            }
        }
        private void ButtonAddAircraftCarrier_Click(object sender, EventArgs e)
        {
            var formAircraftCarrierConfig = new FormAircraftCarrierConfig();
            // TODO Call method AddEvent from formCarConfig
            formAircraftCarrierConfig.AddEvent(AddAircraftCarrier);
            formAircraftCarrierConfig.Show();
        }
        private void AddAircraftCarrier(DrawningAircraftCarrier drawningAircraftCarrier)
        {
            try
            {
                if (listBoxMaps.SelectedIndex == -1)
                {
                    return;
                }
                DrawningObjectAircraftCarrier aircraftcarrier = new(drawningAircraftCarrier);
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + aircraftcarrier != -1)
                {
                    _logger.LogInformation($"Добавлен объект {aircraftcarrier}");
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                }
                else
                {
                    _logger.LogInformation($"Не удалось добавить объект {aircraftcarrier}");
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
            catch (StorageOverflowException ex)
            {
                _logger.LogWarning($"Ошибка переполнения хранилища: {ex.Message}");
                MessageBox.Show($"Ошибка переполнения хранилища: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void buttonRemoveAircraftCarrier_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                var delAircraftCarrier = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos;
                if (delAircraftCarrier != null)
                {
                    _logger.LogInformation($"Объект {delAircraftCarrier} удалён");
                    MessageBox.Show("Объект удален");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                }
                else
                {
                    _logger.LogInformation($"Не удалось удалить объект по позиции {pos}");
                    MessageBox.Show("Не удалось удалить объект");
                }
            }
            catch (AircraftCarrierNotFoundException ex)
            {
                _logger.LogWarning($"Ошибка удаления: {ex.Message}");
                MessageBox.Show($"Ошибка удаления: {ex.Message}");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Неизвестная ошибка: {ex.Message}");
            }
          
        }
        private void buttonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapAircraftCarriersCollectionGeneric.ShowSet();
        }
        private void buttonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }

            pictureBox.Image = _mapAircraftCarriersCollectionGeneric.ShowOnMap();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapAircraftCarriersCollectionGeneric.MoveObject(dir);
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    _logger.LogInformation($"Успешное сохранение в файл {saveFileDialog.FileName}");
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogWarning($"Данные не сохранились: {ex.Message}");
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // TODO продумать логику
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    ReloadMaps();
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Успешная загрузка в файл {openFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    _logger.LogWarning($"Не загрузилось: {ex.Message}");
                    MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}