﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    internal class MapWithSetAircraftCarriersGeneric<T, U>
        where T : class, IDrawningObject
        where U : AbstractMap
    {
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 250;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 100;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetAircraftCarriersGeneric<T> _setAircraftCarriers;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetAircraftCarriersGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setAircraftCarriers = new SetAircraftCarriersGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="car"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetAircraftCarriersGeneric<T, U> map, T AircraftCarrier)
        {
            return map._setAircraftCarriers.Insert(AircraftCarrier);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetAircraftCarriersGeneric<T, U> map, int
       position)
        {
            return map._setAircraftCarriers.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawAircraftCarriers(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();
            for (int i = 0; i < _setAircraftCarriers.Count; i++)
            {
                var aircraftcarrier = _setAircraftCarriers[i];
                if (aircraftcarrier != null)
                {
                    return _map.CreateMap(_pictureWidth, _pictureHeight, aircraftcarrier);
                }
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var aircraftcarrier in _setAircraftCarriers.GetAircraftCarriers())
            {
                data += $"{aircraftcarrier.GetInfo()}{separatorData}";
            }
            return data;
        }
        /// <summary>
        /// Загрузка списка из массива строк
        /// </summary>
        /// <param name="records"></param>
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setAircraftCarriers.Insert(DrawningObjectAircraftCarrier.Create(rec) as T);
            }
        }
        /// <summary>
        /// "Взбалтываем" набор, чтобы все элементы оказались в начале
        /// </summary>
        private void Shaking()
        {
            int j = _setAircraftCarriers.Count - 1;
            for (int i = 0; i < _setAircraftCarriers.Count; i++)
            {
                if (_setAircraftCarriers[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var aircraftcarrier = _setAircraftCarriers[j];
                        if (aircraftcarrier != null)
                        {
                            _setAircraftCarriers.Insert(aircraftcarrier, i);
                            _setAircraftCarriers.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {//линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i *
                   _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth,
               (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawAircraftCarriers(Graphics g)
        {
            int yNumOfPlaces = _pictureHeight / _placeSizeHeight;
            int xNumOfPlaces = _pictureWidth / _placeSizeWidth;

            int rowNum = yNumOfPlaces - 1;
            int columnNum = 0;

            for (int i = 0; i < _setAircraftCarriers.Count; i++)
            {
                if (_setAircraftCarriers[i] != null)
                {
                    (float Left, float Top, float Right, float Bottom) = _setAircraftCarriers[i].GetCurrentPosition();
                    _setAircraftCarriers[i].SetObject(columnNum * _placeSizeWidth, rowNum * _placeSizeHeight + (_placeSizeHeight - (int)(Bottom - Top)), _pictureWidth, _pictureHeight);
                    _setAircraftCarriers[i].DrawningObject(g);
                }
                if (columnNum == xNumOfPlaces - 1)
                {
                    columnNum = 0;
                    rowNum--;
                }
                else
                {
                    columnNum++;
                }
            }
        }
    }
}