﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    internal class MapsCollection
    {
        /// <summary>
        /// Словарь (хранилище) с картами
        /// </summary>
        readonly Dictionary<string, MapWithSetAircraftCarriersGeneric<IDrawningObject,
       AbstractMap>> _mapStorages;
        /// <summary>
        /// Возвращение списка названий карт
        /// </summary>
        public List<string> Keys => _mapStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        private readonly char separatorDict = '|';
        private readonly char separatorData = ';';
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetAircraftCarriersGeneric<IDrawningObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        /// <param name="map">Карта</param>
        public void AddMap(string name, AbstractMap map)
        {
            // TODO Прописать логику для добавления
            if (Keys.Contains(name))
                return;
            _mapStorages.Add(name, new(_pictureWidth, _pictureHeight, map));
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        public void DelMap(string name)
        {
            // TODO Прописать логику для удаления
            _mapStorages.Remove(name);
        }
        /// <summary>
        /// Доступ к парковке
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public MapWithSetAircraftCarriersGeneric<IDrawningObject, AbstractMap> this[string
       ind]
        {
            get
            {
                _mapStorages.TryGetValue(ind, out var mapWithSetAircraftCarriersGeneric);
                return mapWithSetAircraftCarriersGeneric;
            }
        }
        public MapWithSetAircraftCarriersGeneric<IDrawningObject, AbstractMap> this[int index]
        {
            get
            {
                if ((index < 0) || (index >= Keys.Count))
                {
                    return null;
                }

                else
                {
                    string StringKey = Keys[index];
                    return _mapStorages[StringKey];
                }
            }
        }
        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns></returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new(filename))
            {
                sw.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {
                    sw.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }

        }
        /// <summary>
        /// Загрузка нформации по автомобилям на парковках из файла
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader sr = new(filename))
            {
                string str = "";
                if ((str = sr.ReadLine()) == null || !str.Contains("MapsCollection"))
                {
                    //если нет такой записи, то это не те данные
                    throw new FileFormatException("Формат данных в файле неправильный");
                }
                //очищаем записи
                _mapStorages.Clear();
                while ((str = sr.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "HardMap":
                            map = new ComplexMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetAircraftCarriersGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
            }
        }
    }
}