﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    internal class SetAircraftCarriersGeneric<T>
        where T : class
    {
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetAircraftCarriersGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T AircraftCarrier)
        {
            // TODO вставка в начало набора
            return Insert(AircraftCarrier, 0);
        }
        private bool CorrectPos(int pos)
        {
            return 0 <= pos && pos < _maxCount;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T AircraftCarrier, int position)
        {
            // TODO проверка позиции
            if (Count == _maxCount)
            {
                throw new StorageOverflowException(_maxCount);
            }
            if (position < 0 || position > _maxCount) return -1;
            // TODO вставка по позиции
            _places.Insert(position, AircraftCarrier);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            // TODO проверка позиции
            if (position >= Count || position < 0)
            {
                throw new AircraftCarrierNotFoundException(position);
            }
            T aircraftcarrier = _places[position];
            _places.RemoveAt(position);
            return aircraftcarrier;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                return CorrectPos(position) && position < Count ? _places[position] : null;
            }
            set
            {
                Insert(value, position);
            }
           
        }
        public IEnumerable<T> GetAircraftCarriers()
        {
            foreach (var aircraftcarrier in _places)
            {
                if (aircraftcarrier != null)
                {
                    yield return aircraftcarrier;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}